# THIS IS BFTUTORIAL.R WHICH USES A NAMED 
# VECTOR INPUT FOR BAIN TO EVALUATE (INFORMATIVE) HYPOTHESES
#
# THERE IS ALSO AN ASYBFTUTORIAL.R IT USES AN LM OBJECT INPUT FOR
# BAIN TO EVALUATE (INFORMATIVE) HYPOTHESES
#
# BFTUTORIAL.R SHOWS MORE OF THE STEPS INVOLVED IN 
# BAIN HYPOTHESES EVALUATION.
# EASYBFTUTORIAL.R LITERALLY MEANS "MADE AS EASY AS POSSIBLE FOR THE USER"

#***************
#TUTORIAL STEP 1
#***************
# Load libraries. These libraries contain functions such as lm and bain that will be used in 
# this R code. Each time you reopen BFTutorial.R you have to execute this step.

library(MASS)
library(bain)
library(psych)


#***************
#TUTORIAL STEP 2
#***************
# Read Data. If you open monin.txt and holbar.txt in a text editor, you can see
# that variable labels have been inserted between " " in the first line of the file
# (this is called a header).

monin<-read.table("monin.txt",header=TRUE)
monin$group<-factor(monin$group)       # this command tells R that group is a factor and 
# not a continuous variable like attract
holubar<-read.table("holubar.txt",header=TRUE)
holubar$gr <- factor(holubar$gr)       # this command tells R that gr is a factor and 
# not a continuous variable like at


#***************
#TUTORIAL STEP 3
#***************
# Compute descriptives for the Monin data. The dependent variable name is attract, 
# the factor name is group (see also TutorialStep 2).

descrip <- describeBy(monin$attract,monin$group,mat=TRUE)
print(descrip)


#***************
#TUTORIAL STEP 4
#***************
# Compute the Bayes factor for the Monin data. 
# If you return to BFTutorial.R after closing it, you always have to rerun 
# Steps 1 and 2, and the lines between ====== below

#========================================================

# Using the R package lm the means of attract in each of the groups and the residual variance are estimated
# for the Monin data. Note that, attract~group-1 intstructs lm to regress attract on group. The -1 instructs
# lm to estimate the means in each group. Without the -1 regular dummy coding would have been applied. The
# results are collected in, what is called, the R object, prepmonin
prepmonin <-  lm(attract~group-1, data=monin)

# Inspect the estimats and collect the estimates of the group means from prepmonin and store them in the vector estm
estm<-coef(prepmonin)
print(estm)

#assign names to the estimates - seen the bain help file for the requirements that names
#have to adhere to.
names(estm) <- c("g1","g2","g3")

# Collect the residual variance from prepmonin and store it in varm
varm <- (summary(prepmonin)$sigma)**2

# Collect the sample size of each group from the data matrix and store them in the vector sampm
sampm<-table(monin$group)

# Compute the variance of the sample means in each group (divide the residual variance by the sample
# size) and store them in cov1m, cov2m, cov3m
cov1m <- varm/sampm[1]
cov2m <- varm/sampm[2]
cov3m <- varm/sampm[3]

# Tell R that cov1m, cov2m and cov3m are 1x1 matrices
cov1m <- matrix(cov1m,1,1)
cov2m <- matrix(cov2m,1,1)
cov3m <- matrix(cov3m,1,1)

# Collect cov1m, cov2m and cov3m in a list
covm<-list(cov1m,cov2m,cov3m)

#=========================================================

# Run Bain with the information specified thusfar in Tutorial Step 4. Note that, group_parameters=1 states that 
# there is 1 group specific parameter (the mean in each group) and that joint_parameters=0 states that there are
# no parameters that apply to each of the groups (e.g. the regression coefficient of a covariate).
#
# The null-hypothesis that the three means are equal is provided via "g1 = g2 = g3". See the bain help file
# for the requirements that hypotheses have to adhere to.
#
# To be able to repeat an analysis exactly, always set the seed before running bain.
# It is recommended to run analyses with two different seeds to ensure stability of
# the results.
#
# The results of the analysis are stored in the R object resmonin.

set.seed(100)
resmonin<-bain(estm,"g1 = g2 = g3",n=sampm,Sigma=covm,group_parameters=1,joint_parameters = 0)

# print the results of the analysis with bain

print(resmonin)

#***************
#TUTORIAL STEP 5
#***************
# Compute the Bayes factor for "all" the hypotheses for the Monin data
# Note that estm, covm and sampm have already been computed in Step 4.

set.seed(100)
resmonin<-bain(estm,"g1 = g2 = g3; g1 = g2; g1 = g3; g2 = g3",n=sampm,Sigma=covm,group_parameters=1,joint_parameters = 0)

# print the results of the analysis with bain

print(resmonin)

#*****************
#TUTORIAL STEP 6/7
#*****************
# Bayesian updating. Stepwise increasing the sample size for the sesamesim data
# contained in the course pack. This is an alternative for the updating example
# contained in the tutorial paper. It is an extra challenge to see if you have
# digested all that you did previously.

# Open the data file sesamesim30.txt. By doing this you pretend that you are 
# a researcher that has so far collected only a sample containing 30 cases.

# The data concern ability scores of children before (pre) and after (post)
# watching Sesame street for a year. We will work with postnumb (number 
# knowledge after), prenumb (number knowledge before), and age.

# Execute a multiple regression using lm() in which postnumb is predicted from
# prenumb and age. 

# Execute an analysis with bain were the hypotheses are "both regression
# coefficients are equal to zero" and "both regression coefficients" are larger
# than zero.

# Interpret the results. Is the support for one of the hypotheses in your
# opinion large enough? Pretend it is not and that you are going to add
# another 30 cases to your data file (Bayesian updating).

# Repeat the about with sesamesim60.txt, sesamesim90.txt, and sesamesim120.txt.
# This is updating (adding cases to your data file and repeating your analysses).
# What to you conclude?

#****************
#TUTORIAL STEP 8A
#****************
# Sensitivity Analysis Monin Data

# compute BF0a using b-g
set.seed(100)
resmonin<-bain(estm,"g1 = g2 = g3",n=sampm,Sigma=covm,group_parameters=1,joint_parameters = 0)
print(resmonin)

# compute BF0a replacing b-g by 2 x b-g
set.seed(100)
sampm2<- sampm/2 # dividing each sample size by 2 is equivalent to using 2 x b-g
resmonin<-bain(estm,"g1 = g2 = g3",n=sampm2,Sigma=covm,group_parameters=1,joint_parameters = 0)
print(resmonin)

# compute BF0a replacing b-g by 3 x b-g
set.seed(100)
sampm3<- sampm/3 # dividing each sample size by 3 is equivalent to using 3 x b-g
resmonin<-bain(estm,"g1 = g2 = g3",n=sampm3,Sigma=covm,group_parameters=1,joint_parameters = 0)
print(resmonin)

#****************
#TUTORIAL STEP 8B
#****************
# Sensitivity Analysis Holubar Data

prepholubar <-  lm(at~gr-1, data=holubar)
varh <- (summary(prepholubar)$sigma)**2
esth<-coef(prepholubar)
names(esth)<-c("g1", "g2", "g3")

samph<-table(holubar$gr)

cov1h <- varh/samph[1]
cov2h <- varh/samph[2]
cov3h <- varh/samph[3]
cov1h <- matrix(cov1h,1,1)
cov2h <- matrix(cov2h,1,1)
cov3h <- matrix(cov3h,1,1)

covh<-list(cov1h,cov2h,cov3h)

# compute BF0a using b-g
set.seed(100)
resholubar<-bain(esth,"g1 = g2 = g3",n=samph,Sigma=covh,group_parameters=1,joint_parameters = 0)
print(resholubar)

# compute BF0a replacing b-g by 2 x b-g
samph2<- samph/2
set.seed(100)
resholubar<-bain(esth,"g1 = g2 = g3",n=samph2,Sigma=covh,group_parameters=1,joint_parameters = 0)
print(resholubar)

# compute BF0a replacing b-g by 3 x b-g
samph3<- samph/3
set.seed(100)
resholubar<-bain(esth,"g1 = g2 = g3",n=samph3,Sigma=covh,group_parameters=1,joint_parameters = 0)
print(resholubar)

#***************
#TUTORIAL STEP 9
#***************
# Compute the Bayes factor for the Monin data with two outliers

# Created for the third group are two persons with scores of 9 and 10 on attract.
# These have been added to monin.txt rendering the file moninoutlier.txt

monoutl<-read.table("moninoutlier.txt",header=TRUE)
monoutl$group <- factor(monoutl$group)

prepoutlier <-  lm(attract~group-1, data=monoutl)
varoutlier <- (summary(prepoutlier)$sigma)**2
estoutlier<-coef(prepoutlier)
names(estoutlier)<-c("g1", "g2", "g3")


sampoutlier<-table(monoutl$group)

cov1outlier <- varoutlier/sampoutlier[1]
cov2outlier <- varoutlier/sampoutlier[2]
cov3outlier <- varoutlier/sampoutlier[3]
cov1outlier <- matrix(cov1outlier,1,1)
cov2outlier <- matrix(cov2outlier,1,1)
cov3outlier <- matrix(cov3outlier,1,1)

covoutlier<-list(cov1outlier,cov2outlier,cov3outlier)

set.seed(100)
resbain<-bain(estoutlier,"g1 = g2 = g3",n=sampoutlier,Sigma=covoutlier,group_parameters=1,joint_parameters = 0)

# print the results of the analysis with bain

print(resbain)

#****************
#TUTORIAL STEP 10
#****************

# Evaluating informative hypotheses using the Monin data
set.seed(100)
resmonin<-bain(estm,"g1 > g2 > g3; g1 > g2 = g3; g1 = g2 & g1 > g3",n=sampm,Sigma=covm,group_parameters=1,joint_parameters = 0)

# print the results of the analysis with bain
print(resmonin)
# use the command below to obtain the BFmatrix
print(resmonin$BFmatrix)

#****************
#TUTORIAL STEP 11
#****************
# Sensitivity Analyis for the  Evaluation of Inequality Constrained Hypotheses
set.seed(100)
resmonin<-bain(estm,"g1 > g2 > g3",n=sampm,Sigma=covm,group_parameters=1,joint_parameters = 0)
print(resmonin)

sampj2<- sampm/2
set.seed(100)
resmonin<-bain(estm,"g1 > g2 > g3",n=sampj2,Sigma=covm,group_parameters=1,joint_parameters = 0)
print(resmonin)

sampj3<- sampm/3
set.seed(100)
resmonin<-bain(estm,"g1 > g2 > g3",n=sampj3,Sigma=covm,group_parameters=1,joint_parameters = 0)
print(resmonin) 

#*****************
#TUTORIAL STEP 12A
#*****************
# Replication of Monin using the Holubar data

# Compute descriptives the Holubar data. The dependent variable name is at,
# the factor name is gr (see also TutorialStep 2)

descrip <- describeBy(holubar$at,holubar$gr,mat=TRUE)
print(descrip)

#*****************
#TUTORIAL STEP 12B
#*****************
# Evaluate H0 and Ha using the Holubar data - if you interrupted the tutorial
# rerun Tutorial step 8B

set.seed(100)
resholubar<-bain(esth,"g1 = g2 = g3",n=samph,Sigma=covh,group_parameters=1,joint_parameters = 0)
print(resholubar)

#*****************
#TUTORIAL STEP 12C
#*****************
# Evaluate Horiginal as derived from the Monin analysis Using the Holubar data

set.seed(100)
resholubar<-bain(esth,"g1 = g2 = g3; g1 = g2 & g1 > g3",n=samph,Sigma=covh,group_parameters=1,joint_parameters = 0)
print(resholubar)

# To obtain the sample sizes, estimates and covariance matrix run the lines below
print(resholubar$n)
print(resholubar$estimates)
print(resholubar$posterior)

# ********************************************************************
# Below you find the codes used to recreate the monin and holubar data 
# ********************************************************************

# set the seed of the random number generator

# set.seed(123) # note that the seed used to recreate the monin and holubar data 
# used in the tutorial has been lost. This is irrelevant because the sufficient
# statistics of each data set (N, mean, sd) will be the same.

# Create the monin data

# Create variable attract
attract <- 1

# Generate normal random deviation for 67 persons (the Monin sample size)
attract[1:67]<-rnorm(67,mean=0,sd=1)

# For each of the three groups standardize the sampled values
attract[1:19]<-(attract[1:19]-mean(attract[1:19]))/sd(attract[1:19])
attract[20:38]<-(attract[20:38]-mean(attract[20:38]))/sd(attract[20:38])
attract[39:67]<-(attract[39:67]-mean(attract[39:67]))/sd(attract[39:67])

# For each group multiply with the standard deviation and add the mean as reported by Monin
attract[1:19]<-attract[1:19]*1.38+1.88
attract[20:38]<-attract[20:38]*1.95+2.54
attract[39:67]<-attract[39:67]*2.38+.02
group <- 1

# Create a factor group with the numbers 1, 2 and 3
group[1:19]<-1 #"ob"
group[20:38]<-2 #"rsa"
group[39:67]<-3 #"rbog"

# Create a data frame containing a header, attract and group and write it to a file monin.txt
monin<-matrix(c(attract,group),nrow=67,ncol=2)
monin<-as.data.frame(monin)
colnames(monin) <- c("attract", "group")
write.table(monin,"monin.txt",sep=" ",col.names=TRUE,row.names=FALSE)

# create the holubar data

at <- 1
at[1:75]<-rnorm(75,mean=0,sd=1)
at[1:20]<-(at[1:20]-mean(at[1:20]))/sd(at[1:20])
at[21:47]<-(at[21:47]-mean(at[21:47]))/sd(at[21:47])
at[48:75]<-(at[48:75]-mean(at[48:75]))/sd(at[48:75])
at[1:20]<-at[1:20]*1.20+.98
at[21:47]<-at[21:47]*1.88+.02
at[48:75]<-at[48:75]*1.72+.27
gr <- 1
gr[1:20]<-1 #"ob"
gr[21:47]<-2 #"rsa"
gr[48:75]<-3 #"rbog"
holubar<-matrix(c(at,gr),nrow=75,ncol=2)
holubar<-as.data.frame(holubar)
colnames(holubar) <- c("at","gr")
write.table(holubar,"holubar.txt",sep=" ",col.names=TRUE,row.names=FALSE)

